CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  MODE_NORMAL = $0C
  MODE_INVERSE = $0D

  LOCK_DELAY = 50

OBJ
  time : "Timing"
  num : "Numbers"
  spiLock : "Lock"

VAR
  byte ClockPin
  byte DataPin
  byte DCPin
  byte ChipPin
  byte ResetPin

  byte Mode           'Keeps track of current screen mode (inverse or normal)
  
PUB Main | i, StartPin
  InitR(8, 0)

  'Display Test Msg
  str(string("Demo..."))

  i := 65
  
  repeat 60
    cursorxy(0,1)
    PutChar(i++)
    cursorxy(10,2)
    PutChar(i++)
    cursorxy(20,3)
    PutChar(i++)
    cursorxy(30,4)
    PutChar(i++)
    cursorxy(40,5)
    PutChar(i++)
    time.pause1ms(200)    

PUB Init(StartPin)
  ClockPin := StartPin++
  DataPin := StartPin++
  DCPin := StartPin++
  ChipPin := StartPin++
  ResetPin := StartPin

  'Set pins to output
  dira[ClockPin]~~
  dira[DataPin]~~
  dira[DCPin]~~
  dira[ChipPin]~~
  dira[ResetPin]~~   

  InitLCD
  num.Init

PUB InitX(Clock, Data, DC, Chip, Reset, Lock)
  ClockPin := Clock
  DataPin := Data
  DCPin := DC
  ChipPin := Chip
  ResetPin := Reset

  'Set pins to output
  dira[ClockPin]~~
  dira[DataPin]~~
  dira[DCPin]~~
  dira[ChipPin]~~
  dira[ResetPin]~~   

  spiLock.Init(Lock)
  InitLCD
  num.Init

PUB InitR(StartPin, Lock)
  ResetPin := StartPin++
  ChipPin := StartPin++
  DCPin := StartPin++
  DataPin := StartPin++ 
  ClockPin := StartPin
          
  'Set pins to output
  dira[ClockPin]~~
  dira[DataPin]~~
  dira[DCPin]~~
  dira[ChipPin]~~
  dira[ResetPin]~~
    
  
  spiLock.Init(Lock)
  InitLCD
  num.Init

  turnOffShared

PRI InitLCD
  spiLock.getLock(LOCK_DELAY)

  outa[ResetPin] := 1             'set res high
  outa[ChipPin] := 1               'disable chip
  outa[ResetPin] := 0               'set reset low (toggle reset)
  waitcnt(clkfreq / 100 + cnt) 'wait 100ms
  outa[ResetPin] := 1              'wake from reset state
  outa[ChipPin] := 0

  spiLock.clrLock   

  writecom($21)         ' Activate Chip and H=1.
  writecom($C2)         ' Set LCD Voltage to about 7V.
  writecom($13)         ' Adjust voltage bias.
  writecom($20)         ' Horizontal addressing and H=0.
  writecom($09)         ' Activate all segments.

  clear             ' Erase all pixel on the DDRAM.
  
  writecom($08)         ' Blank the Display.    
  writecom($0C)         ' Display Normal.
  Mode := MODE_NORMAL
  cursorxy(0,0) 


PRI turnOffShared

  'outa[ResetPin] := 0             'set res high
  'outa[DataPin] := 0               'disable chip
  'outa[DCPin] := 0               'disable chip
  'outa[ClockPin] := 0               'disable chip

  dira[DataPin] := 0
  dira[DCPin] := 0
  dira[ClockPin] := 0

PRI turnOnShared

  dira[DataPin] := 1
  dira[DCPin] := 1
  dira[ClockPin] := 1

  outa[ClockPin] := 0  

  
PUB str(stringptr)

'' Print a zero-terminated string
  repeat strsize(stringptr)
    PutChar(byte[stringptr++])
    
PUB dec(number)
  str(num.toStr(number, NUM#DEC) + 1)

PUB hex(number)
  str(num.toStr(number, NUM#HEX))

PUB drawImg(img) | i
 ' spiLock.getLock(LOCK_DELAY)

  i := byte[img++]
  repeat(i)
    WriteData(byte[img++])

  'spiLock.clrLock

PUB PutChar(character) | tablept, i

  if(character < $20 | character > $7F)
    return

  tablept := ((5 * character) - 160)

  'spiLock.getLock(LOCK_DELAY)

  repeat i from 5 to 1
    WriteData(Table[tablept])
    tablept++ 

  WriteData($00)   'Space between chars

  'spiLock.clrLock  

PRI WriteCom(toWrite)
  if(DCPin == 0 AND ChipPin == 0)                       'Check that we have been initialized
    return

  spiLock.getLock(LOCK_DELAY)
  turnOnShared
    
  outa[DCPin] := 0 'Command
  outa[ChipPin] := 0 'Turn Chip On
  clockdata(toWrite)
  outa[ChipPin] := 1 'Turn Chip Off
                  
  turnOffShared
  spiLock.clrLock  

PRI WriteData(toWrite)
  if(DCPin == 0 AND ChipPin == 0)
    return

  spiLock.getLock(LOCK_DELAY)
  turnOnShared
    
  outa[DCPin] := 1 'Data             
  outa[ChipPin] := 0 'Turn Chip On
  clockdata(toWrite)
  outa[ChipPin] := 1 'Turn Chip Off   
      
  turnOffShared
  spiLock.clrLock 
 
PUB BlankColumn
  'spiLock.getLock(LOCK_DELAY)
  
  WriteData($00)

  'spiLock.clrLock  

PUB BlankLine(line)
  cursorXY(0,line)
  repeat 84
    BlankColumn
  
PRI ClockData(toSend) | i

  repeat i from 7 to 0
    outa[ClockPin] := 0         'Clock low
    if((toSend >> i) & %1) == 1 'if bit is 1
      outa[DataPin] := 1
    else
      outa[DataPin] := 0

    outa[ClockPin] := 1         'Data is read on rising edge of clock
 

PUB clear | i

  cursorxy(0,0)                 'Cursor home

  'spiLock.getLock(LOCK_DELAY)  
  
  repeat 504
    writedata($00)              'Clear all pixels (Res / 8bits = 504 [(84*48)/8 = 504])

  'spiLock.clrLock
  
PUB CursorXY(X, Y)
  'spiLock.getLock(LOCK_DELAY)
  
  WriteCom($40 | (y & $07))
  WriteCom($80 | (x & $7F))

' spiLock.clrLock

PUB InverseMode
  'spiLock.getLock(LOCK_DELAY)

  if(Mode <> MODE_INVERSE)
    Mode := MODE_INVERSE
    WriteCom(Mode)
'  WriteCom($0D)

  'spiLock.clrLock

PUB NormalMode
 ' spiLock.getLock(LOCK_DELAY)

  if(Mode <> MODE_NORMAL)
    Mode := MODE_NORMAL
    WriteCom(Mode)
'  WriteCom($0C)

  'spiLock.clrLock

PUB GetMode
  return Mode    

Dat
       table byte                                 $00,$00,$00,$00,$00,$00,$00,$5F,$00,$00,                                                        $00,$07,$00,$07,$00,                                                        $14,$7f,$14,$7f,$14,                                                              $24,$2a,$7f,$2a,$12,                                                              $23,$13,$08,$64,$62,                                                             $36,$49,$55,$22,$50,                                                             $00,$05,$03,$00,$00,                                                              $00,$1c,$22,$41,$00,                                                             $00,$41,$22,$1c,$00,                                                             $14,$08,$3e,$08,$14,                                                              $08,$08,$3e,$08,$08,                                                              $00,$50,$30,$00,$00,                                                             $08,$08,$08,$08,$08,                                                              $00,$60,$60,$00,$00,                                                              $20,$10,$08,$04,$02,                                                              $3e,$51,$49,$45,$3e,                                                               $00,$42,$7f,$40,$00,                                                               $42,$61,$51,$49,$46,                                                              $21,$41,$45,$4b,$31,                                                              $18,$14,$12,$7f,$10,                                                              $27,$45,$45,$45,$39,                                                              $3c,$4a,$49,$49,$30,                                                              $01,$71,$09,$05,$03,                                                             $36,$49,$49,$49,$36,                                                              $06,$49,$49,$29,$1e,                                                             $00,$36,$36,$00,$00,                                                             $00,$56,$36,$00,$00,                                                            $08,$14,$22,$41,$00,                                                            $14,$14,$14,$14,$14,                                                            $00,$41,$22,$14,$08,                                                            $02,$01,$51,$09,$06,                                                            $32,$49,$79,$41,$3e,                                                             $7e,$11,$11,$11,$7e,                                                              $7f,$49,$49,$49,$36,                                                              $3e,$41,$41,$41,$22,                                                              $7f,$41,$41,$22,$1c,                                                             $7f,$49,$49,$49,$41,                                                             $7f,$09,$09,$09,$01,                                                             $3e,$41,$49,$49,$7a,                                                             $7f,$08,$08,$08,$7f,                                                             $00,$41,$7f,$41,$00,                                                             $20,$40,$41,$3f,$01,                                                              $7f,$08,$14,$22,$41,                                                              $7f,$40,$40,$40,$40,                                                              $7f,$02,$0c,$02,$7f,                                                              $7f,$04,$08,$10,$7f,                                                              $3e,$41,$41,$41,$3e,                                                              $7f,$09,$09,$09,$06,                                                              $3e,$41,$51,$21,$5e,                                                              $7f,$09,$19,$29,$46,                                                              $46,$49,$49,$49,$31,                                                              $01,$01,$7f,$01,$01,                                                              $3f,$40,$40,$40,$3f,                                                              $1f,$20,$40,$20,$1f,                                                              $3f,$40,$38,$40,$3f,                                                              $63,$14,$08,$14,$63,                                                              $07,$08,$70,$08,$07,                                                              $61,$51,$49,$45,$43,                                                              $00,$7f,$41,$41,$00,                                                              $02,$04,$08,$10,$20,                                                             $00,$41,$41,$7f,$00,                                                              $04,$02,$01,$02,$04,                                                            $40,$40,$40,$40,$40,                                                             $00,$01,$02,$04,$00,                                                             $20,$54,$54,$54,$78,                                                             $7f,$48,$44,$44,$38,                                                             $38,$44,$44,$44,$20,                                                             $38,$44,$44,$48,$7f,                                                             $38,$54,$54,$54,$18,                                                             $08,$7e,$09,$01,$02,                                                             $0c,$52,$52,$52,$3e,                                                             $7f,$08,$04,$04,$78,                                                             $00,$44,$7d,$40,$00,                                                             $20,$40,$44,$3d,$00,                                                              $7f,$10,$28,$44,$00,                                                             $00,$41,$7f,$40,$00,                                                             $7c,$04,$18,$04,$78,                                                             $7c,$08,$04,$04,$78,                                                             $38,$44,$44,$44,$38,                                                             $7c,$14,$14,$14,$08,                                                             $08,$14,$14,$18,$7c,                                                             $7c,$08,$04,$04,$08,                                                             $48,$54,$54,$54,$20,                                                             $04,$3f,$44,$40,$20,                                                             $3c,$40,$40,$20,$7c,                                                             $1c,$20,$40,$20,$1c,                                                             $3c,$40,$30,$40,$3c,                                                             $44,$28,$10,$28,$44,                                                             $0c,$50,$50,$50,$3c,                                                             $44,$64,$54,$4c,$44,                                                             $00,$08,$36,$41,$00,                                                             $00,$00,$7f,$00,$00,                                                             $00,$41,$36,$08,$00,                                                            $10,$08,$08,$10,$08,                                                            $78,$46,$41,$46,$78
      

               